@@ -5,15 +5,13 @@ from typing import List, Optional, Tuple, Union
 
 import numpy as np
 import torch
-import torchaudio
-from packaging.version import parse as _version
 
-from lhotse.utils import Seconds, compute_num_samples, during_docs_build, perturb_num_samples
-
-if not during_docs_build() and _version(torchaudio.__version__) < _version('0.7'):
-    warnings.warn('Torchaudio SoX effects chains are only introduced in version 0.7 - '
-                  'please upgrade your PyTorch to 1.7.1 and torchaudio to 0.7.2 (or higher) '
-                  'to use them.')
+from lhotse.utils import (
+    Seconds,
+    compute_num_samples,
+    during_docs_build,
+    perturb_num_samples,
+)
 
 
 @dataclass
@@ -21,6 +19,7 @@ class RandomValue:
     """
     Represents a uniform distribution in the range [start, end).
     """
+
     start: Union[int, float]
     end: Union[int, float]
 
@@ -58,15 +57,23 @@ class SoxEffectTransform:
         self.effects = effects
 
     def __call__(self, tensor: Union[torch.Tensor, np.ndarray], sampling_rate: int):
+        check_torchaudio_version()
+        import torchaudio
+
         if isinstance(tensor, np.ndarray):
             tensor = torch.from_numpy(tensor)
         effects = self.sample_effects()
-        augmented, new_sampling_rate = torchaudio.sox_effects.apply_effects_tensor(tensor, sampling_rate, effects)
-        assert augmented.shape[0] == tensor.shape[0], "Lhotse does not support modifying the number " \
-                                                      "of channels during data augmentation."
-        assert sampling_rate == new_sampling_rate, \
-            f"Lhotse does not support changing the sampling rate during data augmentation. " \
+        augmented, new_sampling_rate = torchaudio.sox_effects.apply_effects_tensor(
+            tensor, sampling_rate, effects
+        )
+        assert augmented.shape[0] == tensor.shape[0], (
+            "Lhotse does not support modifying the number "
+            "of channels during data augmentation."
+        )
+        assert sampling_rate == new_sampling_rate, (
+            f"Lhotse does not support changing the sampling rate during data augmentation. "
             f"The original SR was '{sampling_rate}', after augmentation it's '{new_sampling_rate}'."
+        )
         # Matching shapes after augmentation -> early return.
         if augmented.shape[1] == tensor.shape[1]:
             return augmented
@@ -74,9 +81,9 @@ class SoxEffectTransform:
         # the WavAugment behavior that we relied upon so far.
         resized = torch.zeros_like(tensor)
         if augmented.shape[1] > tensor.shape[1]:
-            resized = augmented[:, :tensor.shape[1]]
+            resized = augmented[:, : tensor.shape[1]]
         else:
-            resized[:, :augmented.shape[1]] = augmented
+            resized[:, : augmented.shape[1]] = augmented
         return resized
 
     def sample_effects(self) -> List[List[str]]:
@@ -122,6 +129,7 @@ class AudioTransform:
     to instantiate a specific transform object.
     All child classes are expected to be decorated with a ``@dataclass`` decorator.
     """
+
     KNOWN_TRANSFORMS = {}
 
     def __init_subclass__(cls, **kwargs):
@@ -131,12 +139,14 @@ class AudioTransform:
 
     def to_dict(self) -> dict:
         data = asdict(self)
-        return {'name': type(self).__name__, 'kwargs': data}
+        return {"name": type(self).__name__, "kwargs": data}
 
     @staticmethod
-    def from_dict(data: dict) -> 'AudioTransform':
-        assert data['name'] in AudioTransform.KNOWN_TRANSFORMS, f"Unknown transform type: {data['name']}"
-        return AudioTransform.KNOWN_TRANSFORMS[data['name']](**data['kwargs'])
+    def from_dict(data: dict) -> "AudioTransform":
+        assert (
+            data["name"] in AudioTransform.KNOWN_TRANSFORMS
+        ), f"Unknown transform type: {data['name']}"
+        return AudioTransform.KNOWN_TRANSFORMS[data["name"]](**data["kwargs"])
 
     def __call__(self, samples: np.ndarray, sampling_rate: int) -> np.ndarray:
         """
@@ -147,10 +157,7 @@ class AudioTransform:
         raise NotImplementedError
 
     def reverse_timestamps(
-            self,
-            offset: Seconds,
-            duration: Optional[Seconds],
-            sampling_rate: int
+        self, offset: Seconds, duration: Optional[Seconds], sampling_rate: int
     ) -> Tuple[Seconds, Optional[Seconds]]:
         """
         Convert ``offset`` and ``duration`` timestamps to be adequate for the audio before the transform.
@@ -169,21 +176,24 @@ class Speed(AudioTransform):
     It resamples the signal back to the input sampling rate, so the number of output samples will
     be smaller or greater, depending on the speed factor.
     """
+
     factor: float
 
     def __call__(self, samples: np.ndarray, sampling_rate: int) -> np.ndarray:
+        check_torchaudio_version()
+        import torchaudio
+
         sampling_rate = int(sampling_rate)  # paranoia mode
-        effect = [['speed', str(self.factor)], ['rate', str(sampling_rate)]]
+        effect = [["speed", str(self.factor)], ["rate", str(sampling_rate)]]
         if isinstance(samples, np.ndarray):
             samples = torch.from_numpy(samples)
-        augmented, new_sampling_rate = torchaudio.sox_effects.apply_effects_tensor(samples, sampling_rate, effect)
+        augmented, new_sampling_rate = torchaudio.sox_effects.apply_effects_tensor(
+            samples, sampling_rate, effect
+        )
         return augmented.numpy()
 
     def reverse_timestamps(
-            self,
-            offset: Seconds,
-            duration: Optional[Seconds],
-            sampling_rate: int
+        self, offset: Seconds, duration: Optional[Seconds], sampling_rate: int
     ) -> Tuple[Seconds, Optional[Seconds]]:
         """
         This method helps estimate the original offset and duration for a recording
@@ -192,10 +202,21 @@ class Speed(AudioTransform):
         call to ``load_audio()``.
         """
         start_sample = compute_num_samples(offset, sampling_rate)
-        num_samples = compute_num_samples(duration, sampling_rate) if duration is not None else None
+        num_samples = (
+            compute_num_samples(duration, sampling_rate)
+            if duration is not None
+            else None
+        )
         start_sample = perturb_num_samples(start_sample, 1 / self.factor)
-        num_samples = perturb_num_samples(num_samples, 1 / self.factor) if num_samples is not None else None
-        return start_sample / sampling_rate, num_samples / sampling_rate if num_samples is not None else None
+        num_samples = (
+            perturb_num_samples(num_samples, 1 / self.factor)
+            if num_samples is not None
+            else None
+        )
+        return (
+            start_sample / sampling_rate,
+            num_samples / sampling_rate if num_samples is not None else None,
+        )
 
 
 @dataclass
@@ -203,6 +224,7 @@ class Resample(AudioTransform):
     """
     Resampling effect, the same one as invoked with `sox rate` in the command line.
     """
+
     source_sampling_rate: int
     target_sampling_rate: int
 
@@ -212,17 +234,19 @@ class Resample(AudioTransform):
         self.target_sampling_rate = int(self.target_sampling_rate)
 
     def __call__(self, samples: np.ndarray, *args, **kwargs) -> np.ndarray:
-        effect = [['rate', str(self.target_sampling_rate)]]
+        check_torchaudio_version()
+        import torchaudio
+
+        effect = [["rate", str(self.target_sampling_rate)]]
         if isinstance(samples, np.ndarray):
             samples = torch.from_numpy(samples)
-        augmented, _ = torchaudio.sox_effects.apply_effects_tensor(samples, self.source_sampling_rate, effect)
+        augmented, _ = torchaudio.sox_effects.apply_effects_tensor(
+            samples, self.source_sampling_rate, effect
+        )
         return augmented.numpy()
 
     def reverse_timestamps(
-            self,
-            offset: Seconds,
-            duration: Optional[Seconds],
-            sampling_rate: int
+        self, offset: Seconds, duration: Optional[Seconds], sampling_rate: int
     ) -> Tuple[Seconds, Optional[Seconds]]:
         """
         This method helps estimate the original offset and duration for a recording
@@ -236,10 +260,14 @@ class Resample(AudioTransform):
         E.g. 16kHz, 235636 samples correspond to 14.72725s duration; after resampling to 22.05kHz,
         it is 324736 samples which correspond to 14.727256235827664s duration.
         """
-        old_num_samples = compute_num_samples(offset, self.source_sampling_rate, rounding=ROUND_HALF_UP)
+        old_num_samples = compute_num_samples(
+            offset, self.source_sampling_rate, rounding=ROUND_HALF_UP
+        )
         old_offset = old_num_samples / self.source_sampling_rate
         if duration is not None:
-            old_num_samples = compute_num_samples(duration, self.source_sampling_rate, rounding=ROUND_HALF_UP)
+            old_num_samples = compute_num_samples(
+                duration, self.source_sampling_rate, rounding=ROUND_HALF_UP
+            )
             old_duration = old_num_samples / self.source_sampling_rate
         else:
             old_duration = None
@@ -258,6 +286,9 @@ class Tempo(AudioTransform):
     factor: float
 
     def __call__(self, samples: np.ndarray, sampling_rate: int) -> np.ndarray:
+        check_torchaudio_version()
+        import torchaudio
+
         sampling_rate = int(sampling_rate)  # paranoia mode
         if isinstance(samples, np.ndarray):
             samples = torch.from_numpy(samples)
@@ -305,21 +336,27 @@ class Volume(AudioTransform):
     It changes the amplitude of the original samples, so the absolute values of output samples will
     be smaller or greater, depending on the vol factor.
     """
+
     factor: float
 
     def __call__(self, samples: np.ndarray, sampling_rate: int) -> np.ndarray:
+        check_torchaudio_version()
+        import torchaudio
+
         sampling_rate = int(sampling_rate)  # paranoia mode
-        effect = [['vol', str(self.factor)]]
+        effect = [["vol", str(self.factor)]]
         if isinstance(samples, np.ndarray):
             samples = torch.from_numpy(samples)
-        augmented, _ = torchaudio.sox_effects.apply_effects_tensor(samples, sampling_rate, effect)
+        augmented, _ = torchaudio.sox_effects.apply_effects_tensor(
+            samples, sampling_rate, effect
+        )
         return augmented.numpy()
 
     def reverse_timestamps(
-            self,
-            offset: Seconds,
-            duration: Optional[Seconds],
-            sampling_rate: Optional[int] # Not used, made for compatibility purposes
+        self,
+        offset: Seconds,
+        duration: Optional[Seconds],
+        sampling_rate: Optional[int],  # Not used, made for compatibility purposes
     ) -> Tuple[Seconds, Optional[Seconds]]:
         """
         This method just returnes the original offset and duration as volume perturbation
@@ -331,25 +368,43 @@ class Volume(AudioTransform):
 
 def speed(sampling_rate: int) -> List[List[str]]:
     return [
-        ['speed', RandomValue(0.9, 1.1)],
-        ['rate', sampling_rate],  # Resample back to the original sampling rate (speed changes it)
+        ["speed", RandomValue(0.9, 1.1)],
+        [
+            "rate",
+            sampling_rate,
+        ],  # Resample back to the original sampling rate (speed changes it)
     ]
 
 
 def reverb(sampling_rate: int) -> List[List[str]]:
     return [
-        ['reverb', 50, 50, RandomValue(0, 100)],
-        ['remix', '-'],  # Merge all channels (reverb changes mono to stereo)
+        ["reverb", 50, 50, RandomValue(0, 100)],
+        ["remix", "-"],  # Merge all channels (reverb changes mono to stereo)
     ]
 
 
 def volume(sampling_rate: int) -> List[List[str]]:
-    return [['vol', RandomValue(0.125, 2.)]]
+    return [["vol", RandomValue(0.125, 2.0)]]
 
 
 def pitch(sampling_rate: int) -> List[List[str]]:
     return [
         # The returned values are 1/100ths of a semitone, meaning the default is up to a minor third shift up or down.
-        ['pitch', '-q', RandomValue(-300, 300)],
-        ['rate', sampling_rate]  # Resample back to the original sampling rate (pitch changes it)
+        ["pitch", "-q", RandomValue(-300, 300)],
+        [
+            "rate",
+            sampling_rate,
+        ],  # Resample back to the original sampling rate (pitch changes it)
     ]
+
+
+def check_torchaudio_version():
+    import torchaudio
+    from packaging.version import parse as _version
+
+    if not during_docs_build() and _version(torchaudio.__version__) < _version("0.7"):
+        warnings.warn(
+            "Torchaudio SoX effects chains are only introduced in version 0.7 - "
+            "please upgrade your PyTorch to 1.7.1 and torchaudio to 0.7.2 (or higher) "
+            "to use them."
+        )
