@@ -1,11 +1,15 @@
+from .adept import *
 from .aishell import *
+from .aishell4 import *
 from .ami import *
 from .babel import *
 from .broadcast_news import *
 from .callhome_egyptian import *
 from .callhome_english import *
 from .cmu_arctic import *
+from .cmu_indic import *
 from .cmu_kids import *
+from .commonvoice import *
 from .cslu_kids import *
 from .dihard3 import *
 from .fisher_english import *
@@ -14,7 +18,9 @@ from .gale_arabic import *
 from .gale_mandarin import *
 from .gigaspeech import *
 from .heroico import *
+from .hifitts import *
 from .l2_arctic import *
+from .libricss import *
 from .librimix import *
 from .librispeech import *
 from .libritts import *
@@ -25,5 +31,6 @@ from .musan import *
 from .nsc import *
 from .switchboard import *
 from .tedlium import *
+from .timit import *
 from .vctk import *
 from .yesno import *
