@@ -1,5 +1,5 @@
 import random
-from typing import List, Sequence, Union
+from typing import Sequence, Union
 
 from lhotse import CutSet
 
@@ -14,20 +14,24 @@ class PerturbSpeed:
     """
 
     def __init__(
-            self,
-            factors: Union[float, Sequence[float]],
-            p: float,
-            randgen: random.Random = None
+        self,
+        factors: Union[float, Sequence[float]],
+        p: float,
+        randgen: random.Random = None,
+        preserve_id: bool = False,
     ) -> None:
         self.factors = factors if isinstance(factors, Sequence) else [factors]
         self.p = p
         self.random = randgen
+        self.preserve_id = preserve_id
 
     def __call__(self, cuts: CutSet) -> CutSet:
         if self.random is None:
             self.random = random
         return CutSet.from_cuts(
-            cut.perturb_speed(factor=self.random.choice(self.factors))
+            cut.perturb_speed(
+                factor=self.random.choice(self.factors), affix_id=not self.preserve_id
+            )
             if self.random.random() >= self.p
             else cut
             for cut in cuts
