@@ -1,7 +1,7 @@
 import warnings
-from dataclasses import dataclass, field
+from dataclasses import asdict, dataclass, field
 from math import isclose
-from typing import Callable, Iterable, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Iterable, Optional, Tuple, Union
 
 from torch import distributed as dist
 from torch.utils.data import Sampler
@@ -49,12 +49,11 @@ class CutSampler(Sampler):
     """
 
     def __init__(
-            self,
-            shuffle: bool = False,
-            world_size: Optional[int] = None,
-            rank: Optional[int] = None,
-            seed: int = 0,
-            provide_len: bool = True,
+        self,
+        shuffle: bool = False,
+        world_size: Optional[int] = None,
+        rank: Optional[int] = None,
+        seed: int = 0,
     ) -> None:
         """
         :param shuffle: When ``True``, the cuts will be shuffled at the start of iteration.
@@ -64,19 +63,22 @@ class CutSampler(Sampler):
         :param world_size: Total number of distributed nodes. We will try to infer it by default.
         :param rank: Index of distributed node. We will try to infer it by default.
         :param seed: Random seed used to consistently shuffle the dataset across different processes.
-        :param provide_len: Should we expose the ``__len__`` attribute in this class.
-            It makes sense to turn it off when iterating the sampler is somewhat costly for any reason;
-            e.g. because the underlying manifest is lazily loaded from the filesystem/somewhere else.
         """
-        super().__init__(data_source=None)  # the "data_source" arg is not used in Sampler...
+        super().__init__(
+            data_source=None
+        )  # the "data_source" arg is not used in Sampler...
         self.shuffle = shuffle
         self.seed = seed
         self.epoch = 0
-        self.provide_len = provide_len
+
+        # This flag is used to indicate that we have restored a sampler's state from a state_dict.
+        # When it is set, we will ignore the next call to iter(), which would have reset the
+        # iteration state. This way we can resume training exactly from where it was left off.
+        self._just_restored_state = False
 
         self._maybe_init_distributed(world_size=world_size, rank=rank)
-        self.num_batches = None
-        self._filter_fn: Optional[Callable[[Cut], bool]] = None
+        # By default, self._filter_fn passes every Cut through.
+        self._filter_fn: Callable[[Cut], bool] = lambda cut: True
         self.diagnostics = SamplingDiagnostics()
 
     def _maybe_init_distributed(self, world_size: Optional[int], rank: Optional[int]):
@@ -100,6 +102,10 @@ class CutSampler(Sampler):
 
         :param epoch: Epoch number.
         """
+        if self.epoch != epoch:
+            # Changing the epoch automatically tells the sampler to discard the progress
+            # from a previously read state dict.
+            self.allow_iter_to_reset_state()
         self.epoch = epoch
 
     def filter(self, predicate: Callable[[Cut], bool]) -> None:
@@ -109,9 +115,6 @@ class CutSampler(Sampler):
         Can be useful when handling large, lazy manifests where it is not feasible to
         pre-filter them before instantiating the sampler.
 
-        When set, we will remove the ``__len__`` attribute on the sampler, as it is now
-        determined dynamically.
-
         Example:
             >>> cuts = CutSet(...)
             ... sampler = SingleCutSampler(cuts, max_duration=100.0)
@@ -119,7 +122,66 @@ class CutSampler(Sampler):
             ... sampler.filter(lambda cut: 1.0 <= cut.duration <= 20.0)
         """
         self._filter_fn = predicate
-        self.provide_len = False
+
+    def state_dict(self) -> Dict[str, Any]:
+        """
+        Return the current state of the sampler in a state_dict.
+        Together with ``load_state_dict()``, this can be used to restore the
+        training loop's state to the one stored in the state_dict.
+        """
+        return {
+            "epoch": self.epoch,
+            "world_size": self.world_size,
+            "rank": self.rank,
+            "seed": self.seed,
+            "shuffle": self.shuffle,
+            "diagnostics": self.diagnostics.state_dict(),
+        }
+
+    def load_state_dict(self, state_dict: Dict[str, Any]) -> None:
+        """
+        Restore the state of the sampler that is described in a state_dict.
+        This will result in the sampler yielding batches from where the previous training left it off.
+
+        .. caution::
+            The samplers are expected to be initialized with the same CutSets,
+            but this is not explicitly checked anywhere.
+
+        .. caution::
+            The input ``state_dict`` is being mutated: we remove each consumed key, and expect
+            it to be empty at the end of loading. If you don't want this behavior, pass a copy
+            inside of this function (e.g., using ``import deepcopy``).
+
+        .. note::
+            For implementers of sub-classes of CutSampler: the flag ``self._just_restored_state`` has to be
+            handled in ``__iter__`` to make it avoid resetting the just-restored state (only once).
+        """
+        world_size = state_dict.pop("world_size")
+        assert self.world_size == world_size, (
+            f"Cannot restore sampler with a different world_size (before load_state_dict(): {self.world_size},"
+            f"attempted restoring to {world_size}). Changing the world_size would result in different batches "
+            f"being returned from the sampler."
+        )
+        # We are explicitly discarding the "rank" argument to support restoring multi-GPU training
+        # without too much hassle.
+        # We assume that if the world_size is OK, the samplers local ranks are fine.
+        del state_dict["rank"]
+        assert self.seed == state_dict.pop("seed")
+        shuffle = state_dict.pop("shuffle")
+        if self.shuffle != shuffle:
+            warnings.warn(
+                "Overriding the shuffle value in CutSampler based on state_dict"
+                f"(initialized to {self.shuffle}; restored to {shuffle})."
+            )
+        self.shuffle = shuffle
+        self.epoch = state_dict.pop("epoch")
+        self.diagnostics.load_state_dict(state_dict.pop("diagnostics"))
+        assert (
+            len(state_dict) == 0
+        ), "Error in CutSampler.load_state_dict(): Unexpected keys:\n- " + "\n- ".join(
+            state_dict.keys()
+        )
+        self._just_restored_state = True
 
     def __iter__(self):
         raise NotImplementedError(
@@ -138,7 +200,7 @@ class CutSampler(Sampler):
         Not available when the CutSet is read in lazy mode (returns None).
         """
         raise NotImplementedError(
-            'Sub-classes of CutSampler have to implement self.remaining_duration'
+            "Sub-classes of CutSampler have to implement self.remaining_duration"
         )
 
     @property
@@ -148,7 +210,7 @@ class CutSampler(Sampler):
         Not available when the CutSet is read in lazy mode (returns None).
         """
         raise NotImplementedError(
-            'Sub-classes of CutSampler have to implement self.remaining_cuts'
+            "Sub-classes of CutSampler have to implement self.remaining_cuts"
         )
 
     @property
@@ -158,18 +220,20 @@ class CutSampler(Sampler):
         Not available when the CutSet is read in lazy mode (returns None).
         """
         raise NotImplementedError(
-            'Sub-classes of CutSampler have to implement self.num_cuts'
+            "Sub-classes of CutSampler have to implement self.num_cuts"
         )
 
-    def __len__(self) -> int:
-        if not self.provide_len:
-            # Fake non-existence of this attribute
-            raise TypeError(f"object of type '{type(self).__name__}' has no len()")
-        if self.num_batches is None:
-            self.num_batches = sum(1 for _ in self)
-        return self.num_batches
+    def allow_iter_to_reset_state(self):
+        """
+        Enables re-setting to the start of an epoch when iter() is called.
+        This is only needed in one specific scenario: when we restored previous
+        sampler state via ``sampler.load_state_dict()`` but want to discard
+        the progress in the current epoch and start from the beginning.
+        """
+        self._just_restored_state = False
 
     def __next__(self):
+        self.allow_iter_to_reset_state()
         # We use the following trick to ensure equal number of batches for each distributed
         # worker:
         # Every time a next batch is required, we will sample self.world_size batches first,
@@ -268,6 +332,20 @@ class TimeConstraint:
         self.current = 0
         self.num_cuts = 0
 
+    def state_dict(self) -> Dict[str, Any]:
+        return asdict(self)
+
+    def load_state_dict(self, state_dict: Dict[str, Any]) -> None:
+        self.max_duration = state_dict.pop("max_duration")
+        self.max_samples = state_dict.pop("max_samples")
+        self.max_frames = state_dict.pop("max_frames")
+        self.current = state_dict.pop("current")
+        self.num_cuts = state_dict.pop("num_cuts")
+        assert len(state_dict) == 0, (
+            "Error in TimeConstraint.load_state_dict(): Unexpected keys:\n- "
+            + "\n- ".join(state_dict.keys())
+        )
+
     def __add__(self, other: "TimeConstraint") -> "TimeConstraint":
         for key in ("max_duration", "max_frames", "max_samples"):
             self_attr = getattr(self, key)
@@ -322,6 +400,9 @@ class SamplingDiagnostics:
             # We don't warn about discarded batches with 0 cuts.
             self.num_discarded_batches += 1
 
+    def discard_single(self, cut: Cut) -> None:
+        self.discarded_stats.add(cut)
+
     def reset(self) -> None:
         self.kept_stats.reset()
         self.discarded_stats.reset()
@@ -355,11 +436,24 @@ class SamplingDiagnostics:
             f"Overall, {round(self.discarded_stats.current):d} seconds of supervision were discarded."
         )
 
+    def state_dict(self) -> Dict[str, Any]:
+        return asdict(self)
+
+    def load_state_dict(self, state_dict: Dict[str, Any]) -> None:
+        self.num_kept_batches = state_dict.pop("num_kept_batches")
+        self.num_discarded_batches = state_dict.pop("num_discarded_batches")
+        self.kept_stats.load_state_dict(state_dict.pop("kept_stats"))
+        self.discarded_stats.load_state_dict(state_dict.pop("discarded_stats"))
+        assert len(state_dict) == 0, (
+            "Error in SamplingDiagnostics.load_state_dict(): Unexpected keys:\n- "
+            + "\n- ".join(state_dict.keys())
+        )
+
     def __add__(self, other: "SamplingDiagnostics") -> "SamplingDiagnostics":
         return SamplingDiagnostics(
             kept_stats=self.kept_stats + other.kept_stats,
             discarded_stats=self.discarded_stats + other.discarded_stats,
             num_kept_batches=self.num_kept_batches + other.num_kept_batches,
             num_discarded_batches=self.num_discarded_batches
-                                  + other.num_discarded_batches,
+            + other.num_discarded_batches,
         )
