@@ -3,11 +3,33 @@ import random
 from collections import defaultdict
 from dataclasses import dataclass
 from itertools import groupby, islice
-from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Union,
+)
 
 from lhotse.serialization import Serializable
-from lhotse.utils import Pathlike, Seconds, TimeSpan, asdict_nonull, compute_num_samples, exactly_one_not_null, \
-    fastcopy, ifnone, index_by_id_and_check, overspans, perturb_num_samples, split_sequence
+from lhotse.utils import (
+    Pathlike,
+    Seconds,
+    TimeSpan,
+    asdict_nonull,
+    compute_num_samples,
+    exactly_one_not_null,
+    fastcopy,
+    ifnone,
+    index_by_id_and_check,
+    overspans,
+    perturb_num_samples,
+    split_sequence,
+)
 
 
 @dataclass
@@ -22,6 +44,7 @@ class AlignmentItem:
     (see this: https://alexdelorenzo.dev/programming/2018/08/09/bug-in-dataclass.html).
     We can revert to namedtuples if we bump up the Python requirement to 3.8+.
     """
+
     symbol: str
     start: Seconds
     duration: Seconds
@@ -30,11 +53,13 @@ class AlignmentItem:
     def end(self) -> Seconds:
         return round(self.start + self.duration, ndigits=8)
 
-    def with_offset(self, offset: Seconds) -> 'AlignmentItem':
+    def with_offset(self, offset: Seconds) -> "AlignmentItem":
         """Return an identical ``AlignmentItem``, but with the ``offset`` added to the ``start`` field."""
-        return AlignmentItem(self.symbol, round(self.start + offset, ndigits=8), self.duration)
+        return AlignmentItem(
+            self.symbol, round(self.start + offset, ndigits=8), self.duration
+        )
 
-    def perturb_speed(self, factor: float, sampling_rate: int) -> 'AlignmentItem':
+    def perturb_speed(self, factor: float, sampling_rate: int) -> "AlignmentItem":
         """
         Return an ``AlignmentItem`` that has time boundaries matching the
         recording/cut perturbed with the same factor. See :meth:`SupervisionSegment.perturb_speed`
@@ -46,16 +71,20 @@ class AlignmentItem:
         new_duration = perturb_num_samples(num_samples, factor) / sampling_rate
         return AlignmentItem(self.symbol, new_start, new_duration)
 
-    def trim(self, end: Seconds, start: Seconds = 0) -> 'AlignmentItem':
+    def trim(self, end: Seconds, start: Seconds = 0) -> "AlignmentItem":
         """
         See :met:`SupervisionSegment.trim`.
         """
         assert start >= 0
         start_exceeds_by = abs(min(0, self.start - start))
         end_exceeds_by = max(0, self.end - end)
-        return AlignmentItem(self.symbol, max(start, self.start), self.duration - end_exceeds_by - start_exceeds_by)
+        return AlignmentItem(
+            self.symbol,
+            max(start, self.start),
+            self.duration - end_exceeds_by - start_exceeds_by,
+        )
 
-    def transform(self, transform_fn: Callable[[str], str]) -> 'AlignmentItem':
+    def transform(self, transform_fn: Callable[[str], str]) -> "AlignmentItem":
         """
         Perform specified transformation on the alignment content.
         """
@@ -141,6 +170,7 @@ class SupervisionSegment:
 
 
     """
+
     id: str
     recording_id: str
     start: Seconds
@@ -157,7 +187,7 @@ class SupervisionSegment:
     def end(self) -> Seconds:
         return round(self.start + self.duration, ndigits=8)
 
-    def with_offset(self, offset: Seconds) -> 'SupervisionSegment':
+    def with_offset(self, offset: Seconds) -> "SupervisionSegment":
         """Return an identical ``SupervisionSegment``, but with the ``offset`` added to the ``start`` field."""
         return SupervisionSegment(
             id=self.id,
@@ -171,20 +201,16 @@ class SupervisionSegment:
             gender=self.gender,
             custom=self.custom,
             alignment={
-                type: [
-                    item.with_offset(offset=offset)
-                    for item in ali
-                ]
+                type: [item.with_offset(offset=offset) for item in ali]
                 for type, ali in self.alignment.items()
-            } if self.alignment else None
+            }
+            if self.alignment
+            else None,
         )
 
     def perturb_speed(
-            self,
-            factor: float,
-            sampling_rate: int,
-            affix_id: bool = True
-    ) -> 'SupervisionSegment':
+        self, factor: float, sampling_rate: int, affix_id: bool = True
+    ) -> "SupervisionSegment":
         """
         Return a ``SupervisionSegment`` that has time boundaries matching the
         recording/cut perturbed with the same factor.
@@ -202,25 +228,26 @@ class SupervisionSegment:
         new_duration = perturb_num_samples(num_samples, factor) / sampling_rate
         return fastcopy(
             self,
-            id=f'{self.id}_sp{factor}' if affix_id else self.id,
-            recording_id=f'{self.recording_id}_sp{factor}' if affix_id else self.recording_id,
+            id=f"{self.id}_sp{factor}" if affix_id else self.id,
+            recording_id=f"{self.recording_id}_sp{factor}"
+            if affix_id
+            else self.recording_id,
             start=new_start,
             duration=new_duration,
             alignment={
-                type:[
+                type: [
                     item.perturb_speed(factor=factor, sampling_rate=sampling_rate)
                     for item in ali
                 ]
                 for type, ali in self.alignment.items()
-            } if self.alignment else None
+            }
+            if self.alignment
+            else None,
         )
 
     def perturb_tempo(
-            self,
-            factor: float,
-            sampling_rate: int,
-            affix_id: bool = True
-    ) -> 'SupervisionSegment':
+        self, factor: float, sampling_rate: int, affix_id: bool = True
+    ) -> "SupervisionSegment":
         """
         Return a ``SupervisionSegment`` that has time boundaries matching the
         recording/cut perturbed with the same factor.
@@ -237,15 +264,15 @@ class SupervisionSegment:
         perturbed = self.perturb_speed(factor, sampling_rate, affix_id=False)
         return fastcopy(
             perturbed,
-            id=f'{self.id}_tp{factor}' if affix_id else self.id,
-            recording_id=f'{self.recording_id}_tp{factor}' if affix_id else self.recording_id,
+            id=f"{self.id}_tp{factor}" if affix_id else self.id,
+            recording_id=f"{self.recording_id}_tp{factor}"
+            if affix_id
+            else self.recording_id,
         )
 
     def perturb_volume(
-            self,
-            factor: float,
-            affix_id: bool = True
-    ) -> 'SupervisionSegment':
+        self, factor: float, affix_id: bool = True
+    ) -> "SupervisionSegment":
         """
         Return a ``SupervisionSegment`` with modified ids.
 
@@ -257,11 +284,13 @@ class SupervisionSegment:
 
         return fastcopy(
             self,
-            id=f'{self.id}_vp{factor}' if affix_id else self.id,
-            recording_id=f'{self.recording_id}_vp{factor}' if affix_id else self.recording_id
+            id=f"{self.id}_vp{factor}" if affix_id else self.id,
+            recording_id=f"{self.recording_id}_vp{factor}"
+            if affix_id
+            else self.recording_id,
         )
 
-    def trim(self, end: Seconds, start: Seconds = 0) -> 'SupervisionSegment':
+    def trim(self, end: Seconds, start: Seconds = 0) -> "SupervisionSegment":
         """
         Return an identical ``SupervisionSegment``, but ensure that ``self.start`` is not negative (in which case
         it's set to 0) and ``self.end`` does not exceed the ``end`` parameter. If a `start` is optionally
@@ -278,15 +307,16 @@ class SupervisionSegment:
             start=max(start, self.start),
             duration=self.duration - end_exceeds_by - start_exceeds_by,
             alignment={
-                type:[
-                    item.trim(end=end, start=start)
-                    for item in ali
-                ]
+                type: [item.trim(end=end, start=start) for item in ali]
                 for type, ali in self.alignment.items()
-            } if self.alignment else None
+            }
+            if self.alignment
+            else None,
         )
 
-    def map(self, transform_fn: Callable[['SupervisionSegment'], 'SupervisionSegment']) -> 'SupervisionSegment':
+    def map(
+        self, transform_fn: Callable[["SupervisionSegment"], "SupervisionSegment"]
+    ) -> "SupervisionSegment":
         """
         Return a copy of the current segment, transformed with ``transform_fn``.
 
@@ -295,7 +325,9 @@ class SupervisionSegment:
         """
         return transform_fn(self)
 
-    def transform_text(self, transform_fn: Callable[[str], str]) -> 'SupervisionSegment':
+    def transform_text(
+        self, transform_fn: Callable[[str], str]
+    ) -> "SupervisionSegment":
         """
         Return a copy of the current segment with transformed ``text`` field.
         Useful for text normalization, phonetic transcription, etc.
@@ -308,10 +340,8 @@ class SupervisionSegment:
         return fastcopy(self, text=transform_fn(self.text))
 
     def transform_alignment(
-        self,
-        transform_fn: Callable[[str], str],
-        type: Optional[str] = 'word'
-    ) -> 'SupervisionSegment':
+        self, transform_fn: Callable[[str], str], type: Optional[str] = "word"
+    ) -> "SupervisionSegment":
         """
         Return a copy of the current segment with transformed ``alignment`` field.
         Useful for text normalization, phonetic transcription, etc.
@@ -325,25 +355,28 @@ class SupervisionSegment:
         return fastcopy(
             self,
             alignment={
-                ali_type:[
+                ali_type: [
                     item.transform(transform_fn=transform_fn)
-                    if ali_type == type else item
+                    if ali_type == type
+                    else item
                     for item in ali
                 ]
                 for ali_type, ali in self.alignment.items()
-            }
+            },
         )
 
     def to_dict(self) -> dict:
         return asdict_nonull(self)
 
     @staticmethod
-    def from_dict(data: dict) -> 'SupervisionSegment':
+    def from_dict(data: dict) -> "SupervisionSegment":
         return SupervisionSegment(
             **{
-                key:(
-                    {k:[AlignmentItem(**x) for x in v] for k,v in value.items()}
-                    if key == 'alignment' else value)
+                key: (
+                    {k: [AlignmentItem(**x) for x in v] for k, v in value.items()}
+                    if key == "alignment"
+                    else value
+                )
                 for key, value in data.items()
             }
         )
@@ -398,7 +431,7 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
     def __init__(self, segments: Mapping[str, SupervisionSegment]) -> None:
         self.segments = ifnone(segments, {})
 
-    def __eq__(self, other: 'SupervisionSet') -> bool:
+    def __eq__(self, other: "SupervisionSet") -> bool:
         return self.segments == other.segments
 
     @property
@@ -407,6 +440,7 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         Indicates whether this manifest was opened in lazy (read-on-the-fly) mode or not.
         """
         from lhotse.serialization import LazyJsonlIterator
+
         return isinstance(self.segments, LazyJsonlIterator)
 
     @property
@@ -414,19 +448,18 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         return self.segments.keys()
 
     @staticmethod
-    def from_segments(segments: Iterable[SupervisionSegment]) -> 'SupervisionSet':
+    def from_segments(segments: Iterable[SupervisionSegment]) -> "SupervisionSet":
         return SupervisionSet(segments=index_by_id_and_check(segments))
 
     @staticmethod
-    def from_dicts(data: Iterable[Dict]) -> 'SupervisionSet':
-        return SupervisionSet.from_segments(SupervisionSegment.from_dict(s) for s in data)
+    def from_dicts(data: Iterable[Dict]) -> "SupervisionSet":
+        return SupervisionSet.from_segments(
+            SupervisionSegment.from_dict(s) for s in data
+        )
 
     def with_alignment_from_ctm(
-            self,
-            ctm_file: Pathlike,
-            type: str = 'word',
-            match_channel: bool = False
-    ) -> 'SupervisionSet':
+        self, ctm_file: Pathlike, type: str = "word", match_channel: bool = False
+    ) -> "SupervisionSet":
         """
         Add alignments from CTM file to the supervision set.
 
@@ -439,44 +472,54 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         with open(ctm_file) as f:
             for line in f:
                 reco_id, channel, start, duration, symbol = line.strip().split()
-                ctm_words.append((reco_id, int(channel), float(start), float(duration), symbol))
-        ctm_words = sorted(ctm_words, key=lambda x:(x[0], x[2]))
-        reco_to_ctm = defaultdict(list, {k: list(v) for k,v in groupby(ctm_words, key=lambda x:x[0])})
+                ctm_words.append(
+                    (reco_id, int(channel), float(start), float(duration), symbol)
+                )
+        ctm_words = sorted(ctm_words, key=lambda x: (x[0], x[2]))
+        reco_to_ctm = defaultdict(
+            list, {k: list(v) for k, v in groupby(ctm_words, key=lambda x: x[0])}
+        )
         segments = []
         num_total = len(ctm_words)
         num_overspanned = 0
         for reco_id in set([s.recording_id for s in self]):
             if reco_id in reco_to_ctm:
                 for seg in self.find(recording_id=reco_id):
-                    alignment = [AlignmentItem(symbol=word[4], start=word[2], duration=word[3]) for word in reco_to_ctm[reco_id]
-                                    if overspans(seg, TimeSpan(word[2], word[2] + word[3]))
-                                    and (seg.channel == word[1] or not match_channel)
-                                ]
+                    alignment = [
+                        AlignmentItem(symbol=word[4], start=word[2], duration=word[3])
+                        for word in reco_to_ctm[reco_id]
+                        if overspans(seg, TimeSpan(word[2], word[2] + word[3]))
+                        and (seg.channel == word[1] or not match_channel)
+                    ]
                     num_overspanned += len(alignment)
                     segments.append(fastcopy(seg, alignment={type: alignment}))
             else:
                 segments.append([s for s in self.find(recording_id=reco_id)])
-        logging.info(f"{num_overspanned} alignments added out of {num_total} total. If there are several"
-            " missing, there could be a mismatch problem.")
+        logging.info(
+            f"{num_overspanned} alignments added out of {num_total} total. If there are several"
+            " missing, there could be a mismatch problem."
+        )
         return SupervisionSet.from_segments(segments)
 
-    def write_alignment_to_ctm(self, ctm_file: Pathlike, type: str = 'word') -> None:
+    def write_alignment_to_ctm(self, ctm_file: Pathlike, type: str = "word") -> None:
         """
         Write alignments to CTM file.
 
         :param ctm_file: Path to output CTM file (will be created if not exists)
         :param type: Alignment type to write (default = `word`)
         """
-        with open(ctm_file, 'w') as f:
+        with open(ctm_file, "w") as f:
             for s in self:
                 if type in s.alignment:
                     for ali in s.alignment[type]:
-                        f.write(f'{s.recording_id} {s.channel} {ali.start:.02f} {ali.duration:.02f} {ali.symbol}\n')
+                        f.write(
+                            f"{s.recording_id} {s.channel} {ali.start:.02f} {ali.duration:.02f} {ali.symbol}\n"
+                        )
 
     def to_dicts(self) -> Iterable[dict]:
         return (s.to_dict() for s in self)
 
-    def shuffle(self, rng: Optional[random.Random] = None) -> 'SupervisionSet':
+    def shuffle(self, rng: Optional[random.Random] = None) -> "SupervisionSet":
         """
         Shuffle the supervision IDs in the current :class:`.SupervisionSet` and return a shuffled copy of self.
 
@@ -489,7 +532,9 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         rng.shuffle(ids)
         return SupervisionSet(segments={sid: self[sid] for sid in ids})
 
-    def split(self, num_splits: int, shuffle: bool = False, drop_last: bool = False) -> List['SupervisionSet']:
+    def split(
+        self, num_splits: int, shuffle: bool = False, drop_last: bool = False
+    ) -> List["SupervisionSet"]:
         """
         Split the :class:`~lhotse.SupervisionSet` into ``num_splits`` pieces of equal size.
 
@@ -502,11 +547,15 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         :return: A list of :class:`~lhotse.SupervisionSet` pieces.
         """
         return [
-            SupervisionSet.from_segments(subset) for subset in
-            split_sequence(self, num_splits=num_splits, shuffle=shuffle, drop_last=drop_last)
+            SupervisionSet.from_segments(subset)
+            for subset in split_sequence(
+                self, num_splits=num_splits, shuffle=shuffle, drop_last=drop_last
+            )
         ]
 
-    def subset(self, first: Optional[int] = None, last: Optional[int] = None) -> 'SupervisionSet':
+    def subset(
+        self, first: Optional[int] = None, last: Optional[int] = None
+    ) -> "SupervisionSet":
         """
         Return a new ``SupervisionSet`` according to the selected subset criterion.
         Only a single argument to ``subset`` is supported at this time.
@@ -515,25 +564,35 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         :param last: int, the number of last supervisions to keep.
         :return: a new ``SupervisionSet`` with the subset results.
         """
-        assert exactly_one_not_null(first, last), "subset() can handle only one non-None arg."
+        assert exactly_one_not_null(
+            first, last
+        ), "subset() can handle only one non-None arg."
 
         if first is not None:
             assert first > 0
             if first > len(self):
-                logging.warning(f'SupervisionSet has only {len(self)} items but first {first} required; '
-                                f'not doing anything.')
+                logging.warning(
+                    f"SupervisionSet has only {len(self)} items but first {first} required; "
+                    f"not doing anything."
+                )
                 return self
             return SupervisionSet.from_segments(islice(self, first))
 
         if last is not None:
             assert last > 0
             if last > len(self):
-                logging.warning(f'SupervisionSet has only {len(self)} items but last {last} required; '
-                                f'not doing anything.')
+                logging.warning(
+                    f"SupervisionSet has only {len(self)} items but last {last} required; "
+                    f"not doing anything."
+                )
                 return self
-            return SupervisionSet.from_segments(islice(self, len(self) - last, len(self)))
+            return SupervisionSet.from_segments(
+                islice(self, len(self) - last, len(self))
+            )
 
-    def filter(self, predicate: Callable[[SupervisionSegment], bool]) -> 'SupervisionSet':
+    def filter(
+        self, predicate: Callable[[SupervisionSegment], bool]
+    ) -> "SupervisionSet":
         """
         Return a new SupervisionSet with the SupervisionSegments that satisfy the `predicate`.
 
@@ -542,7 +601,9 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         """
         return SupervisionSet.from_segments(seg for seg in self if predicate(seg))
 
-    def map(self, transform_fn: Callable[[SupervisionSegment], SupervisionSegment]) -> 'SupervisionSet':
+    def map(
+        self, transform_fn: Callable[[SupervisionSegment], SupervisionSegment]
+    ) -> "SupervisionSet":
         """
         Map a ``transform_fn`` to the SupervisionSegments and return a new ``SupervisionSet``.
 
@@ -551,7 +612,7 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         """
         return SupervisionSet.from_segments(s.map(transform_fn) for s in self)
 
-    def transform_text(self, transform_fn: Callable[[str], str]) -> 'SupervisionSet':
+    def transform_text(self, transform_fn: Callable[[str], str]) -> "SupervisionSet":
         """
         Return a copy of the current ``SupervisionSet`` with the segments having a transformed ``text`` field.
         Useful for text normalization, phonetic transcription, etc.
@@ -559,9 +620,13 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         :param transform_fn: a function that accepts a string and returns a string.
         :return: a ``SupervisionSet`` with adjusted text.
         """
-        return SupervisionSet.from_segments(s.transform_text(transform_fn) for s in self)
+        return SupervisionSet.from_segments(
+            s.transform_text(transform_fn) for s in self
+        )
 
-    def transform_alignment(self, transform_fn: Callable[[str], str], type: str = 'word') -> 'SupervisionSet':
+    def transform_alignment(
+        self, transform_fn: Callable[[str], str], type: str = "word"
+    ) -> "SupervisionSet":
         """
         Return a copy of the current ``SupervisionSet`` with the segments having a transformed ``alignment`` field.
         Useful for text normalization, phonetic transcription, etc.
@@ -570,16 +635,18 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         :param type:  alignment type to transform (key for alignment dict).
         :return: a ``SupervisionSet`` with adjusted text.
         """
-        return SupervisionSet.from_segments(s.transform_alignment(transform_fn, type=type) for s in self)
+        return SupervisionSet.from_segments(
+            s.transform_alignment(transform_fn, type=type) for s in self
+        )
 
     def find(
-            self,
-            recording_id: str,
-            channel: Optional[int] = None,
-            start_after: Seconds = 0,
-            end_before: Optional[Seconds] = None,
-            adjust_offset: bool = False,
-            tolerance: Seconds = 0.001
+        self,
+        recording_id: str,
+        channel: Optional[int] = None,
+        start_after: Seconds = 0,
+        end_before: Optional[Seconds] = None,
+        adjust_offset: bool = False,
+        tolerance: Seconds = 0.001,
     ) -> Iterable[SupervisionSegment]:
         """
         Return an iterable of segments that match the provided ``recording_id``.
@@ -606,8 +673,8 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
             segment.with_offset(-start_after) if adjust_offset else segment
             for segment in segment_by_recording_id.get(recording_id, [])
             if (channel is None or segment.channel == channel)
-               and segment.start >= start_after - tolerance
-               and (end_before is None or segment.end <= end_before + tolerance)
+            and segment.start >= start_after - tolerance
+            and (end_before is None or segment.end <= end_before + tolerance)
         )
 
     # This is a cache that significantly speeds up repeated ``find()`` queries.
@@ -616,17 +683,22 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
     def _index_by_recording_id_and_cache(self):
         if self._segments_by_recording_id is None:
             from cytoolz import groupby
+
             self._segments_by_recording_id = groupby(lambda seg: seg.recording_id, self)
         return self._segments_by_recording_id
 
     def __repr__(self) -> str:
-        return f'SupervisionSet(len={len(self)})'
+        return f"SupervisionSet(len={len(self)})"
 
     def __getitem__(self, sup_id_or_index: Union[int, str]) -> SupervisionSegment:
         if isinstance(sup_id_or_index, str):
             return self.segments[sup_id_or_index]
         # ~100x faster than list(dict.values())[index] for 100k elements
-        return next(val for idx, val in enumerate(self.segments.values()) if idx == sup_id_or_index)
+        return next(
+            val
+            for idx, val in enumerate(self.segments.values())
+            if idx == sup_id_or_index
+        )
 
     def __contains__(self, item: Union[str, SupervisionSegment]) -> bool:
         if isinstance(item, str):
@@ -640,5 +712,5 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
     def __len__(self) -> int:
         return len(self.segments)
 
-    def __add__(self, other: 'SupervisionSet') -> 'SupervisionSet':
+    def __add__(self, other: "SupervisionSet") -> "SupervisionSet":
         return SupervisionSet(segments={**self.segments, **other.segments})
