@@ -1,13 +1,24 @@
+import logging
 import math
 import random
 import uuid
-import logging
 from contextlib import AbstractContextManager, contextmanager
 from dataclasses import asdict, dataclass
 from decimal import Decimal, ROUND_HALF_DOWN, ROUND_HALF_UP
 from math import ceil, isclose
 from pathlib import Path
-from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, TypeVar, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import numpy as np
 import torch
@@ -15,7 +26,7 @@ from tqdm.auto import tqdm
 from typing_extensions import Literal
 
 Pathlike = Union[Path, str]
-T = TypeVar('T')
+T = TypeVar("T")
 
 Seconds = float
 Decibels = float
@@ -62,39 +73,56 @@ class SmartOpen:
         >>>     SmartOpen.open(s3_url, 'rb') as f:
         >>>         source = f.read()
     """
+
     transport_params: Optional[Dict] = None
     compression: Optional[str] = None
-    import_err_msg = ("Please do 'pip install smart_open' - "
-                      "if you are using S3/GCP/Azure/other cloud-specific URIs, do "
-                      "'pip install smart_open[s3]' (or smart_open[gcp], etc.) instead.")
+    import_err_msg = (
+        "Please do 'pip install smart_open' - "
+        "if you are using S3/GCP/Azure/other cloud-specific URIs, do "
+        "'pip install smart_open[s3]' (or smart_open[gcp], etc.) instead."
+    )
     smart_open: Optional[Callable] = None
 
     @classmethod
     def setup(
-            cls,
-            compression: Optional[str]=None,
-            transport_params: Optional[dict]= None):
+        cls, compression: Optional[str] = None, transport_params: Optional[dict] = None
+    ):
         try:
             from smart_open import open as sm_open
         except ImportError:
             raise ImportError(cls.import_err_msg)
-        if cls.transport_params is not None and cls.transport_params != transport_params:
-            logging.warning(f'SmartOpen.setup second call overwrites existing transport_params with new version'
-                    f'\t\n{cls.transport_params}\t\nvs\t\n{transport_params}')
+        if (
+            cls.transport_params is not None
+            and cls.transport_params != transport_params
+        ):
+            logging.warning(
+                f"SmartOpen.setup second call overwrites existing transport_params with new version"
+                f"\t\n{cls.transport_params}\t\nvs\t\n{transport_params}"
+            )
         if cls.compression is not None and cls.compression != compression:
-            logging.warning(f'SmartOpen.setup second call overwrites existing compression param with new version'
-                    f'\t\n{cls.compression} vs {compression}')
+            logging.warning(
+                f"SmartOpen.setup second call overwrites existing compression param with new version"
+                f"\t\n{cls.compression} vs {compression}"
+            )
         cls.transport_params = transport_params
         cls.compression = compression
         cls.smart_open = sm_open
 
     @classmethod
-    def open(cls, uri, mode='rb', compression=None, transport_params=None, **kwargs):
+    def open(cls, uri, mode="rb", compression=None, transport_params=None, **kwargs):
         if cls.smart_open is None:
             cls.setup(compression=compression, transport_params=transport_params)
         compression = compression if compression else cls.compression
-        transport_params = transport_params if transport_params else cls.transport_params
-        return cls.smart_open(uri, mode=mode, compression=compression, transport_params=transport_params, **kwargs)
+        transport_params = (
+            transport_params if transport_params else cls.transport_params
+        )
+        return cls.smart_open(
+            uri,
+            mode=mode,
+            compression=compression,
+            transport_params=transport_params,
+            **kwargs,
+        )
 
 
 def fix_random_seed(random_seed: int):
@@ -127,6 +155,7 @@ def asdict_nonull(dclass) -> Dict[str, Any]:
     Recursively convert a dataclass into a dict, removing all the fields with `None` value.
     Intended to use in place of dataclasses.asdict(), when the null values are not desired in the serialized document.
     """
+
     def non_null_dict_factory(collection):
         d = dict(collection)
         remove_keys = []
@@ -151,6 +180,7 @@ class SetContainingAnything:
 @dataclass
 class TimeSpan:
     """Helper class for specifying a time span."""
+
     start: Seconds
     end: Seconds
 
@@ -158,8 +188,12 @@ class TimeSpan:
 # TODO: Ugh, Protocols are only in Python 3.8+...
 def overlaps(lhs: Any, rhs: Any) -> bool:
     """Indicates whether two time-spans/segments are overlapping or not."""
-    return lhs.start < rhs.end and rhs.start < lhs.end \
-           and not isclose(lhs.start, rhs.end) and not isclose(rhs.start, lhs.end)
+    return (
+        lhs.start < rhs.end
+        and rhs.start < lhs.end
+        and not isclose(lhs.start, rhs.end)
+        and not isclose(rhs.start, lhs.end)
+    )
 
 
 def overspans(spanning: Any, spanned: Any) -> bool:
@@ -167,14 +201,18 @@ def overspans(spanning: Any, spanned: Any) -> bool:
     return spanning.start <= spanned.start <= spanned.end <= spanning.end
 
 
-def time_diff_to_num_frames(time_diff: Seconds, frame_length: Seconds, frame_shift: Seconds) -> int:
+def time_diff_to_num_frames(
+    time_diff: Seconds, frame_length: Seconds, frame_shift: Seconds
+) -> int:
     """Convert duration to an equivalent number of frames, so as to not exceed the duration."""
     if isclose(time_diff, 0.0):
         return 0
     return int(ceil((time_diff - frame_length) / frame_shift))
 
 
-def check_and_rglob(path: Pathlike, pattern: str, strict: Optional[bool] = True) -> List[Path]:
+def check_and_rglob(
+    path: Pathlike, pattern: str, strict: Optional[bool] = True
+) -> List[Path]:
     """
     Asserts that ``path`` exists, is a directory and contains at least one file satisfying the ``pattern``.
     If `strict` is False, then zero matches are allowed.
@@ -182,9 +220,11 @@ def check_and_rglob(path: Pathlike, pattern: str, strict: Optional[bool] = True)
     :returns: a list of paths to files matching the ``pattern``.
     """
     path = Path(path)
-    assert path.is_dir(), f'No such directory: {path}'
+    assert path.is_dir(), f"No such directory: {path}"
     matches = sorted(path.rglob(pattern))
-    assert len(matches) > 0 or not strict, f'No files matching pattern "{pattern}" in directory: {path}'
+    assert (
+        len(matches) > 0 or not strict
+    ), f'No files matching pattern "{pattern}" in directory: {path}'
     return matches
 
 
@@ -199,6 +239,7 @@ def recursion_limit(stack_size: int):
         >>>     pass
     """
     import sys
+
     old_size = sys.getrecursionlimit()
     sys.setrecursionlimit(stack_size)
     try:
@@ -222,10 +263,7 @@ def fastcopy(dataclass_obj: T, **kwargs) -> T:
 
 
 def split_sequence(
-        seq: Sequence[Any],
-        num_splits: int,
-        shuffle: bool = False,
-        drop_last: bool = False
+    seq: Sequence[Any], num_splits: int, shuffle: bool = False, drop_last: bool = False
 ) -> List[List[Any]]:
     """
     Split a sequence into ``num_splits`` equal parts. The element order can be randomized.
@@ -243,7 +281,9 @@ def split_sequence(
     seq = list(seq)
     num_items = len(seq)
     if num_splits > num_items:
-        raise ValueError(f"Cannot split iterable into more chunks ({num_splits}) than its number of items {num_items}")
+        raise ValueError(
+            f"Cannot split iterable into more chunks ({num_splits}) than its number of items {num_items}"
+        )
     if shuffle:
         random.shuffle(seq)
     chunk_size = num_items // num_splits
@@ -259,21 +299,25 @@ def split_sequence(
         # [10, 20] -> [11, 22]  (begin_shift=1, end_shift=2)
         # [20, 30] -> [22, 32]  (begin_shift=2, end_shift=2)
         # for num_items=32 and num_splits=3
-        end_shifts = list(range(1, num_shifts + 1)) + [num_shifts] * (num_splits - num_shifts)
+        end_shifts = list(range(1, num_shifts + 1)) + [num_shifts] * (
+            num_splits - num_shifts
+        )
         begin_shifts = [0] + end_shifts[:-1]
 
     split_indices = [
         [i * chunk_size + begin_shift, (i + 1) * chunk_size + end_shift]
-        for i, begin_shift, end_shift in zip(range(num_splits), begin_shifts, end_shifts)
+        for i, begin_shift, end_shift in zip(
+            range(num_splits), begin_shifts, end_shifts
+        )
     ]
     splits = [seq[begin:end] for begin, end in split_indices]
     return splits
 
 
 def compute_num_frames(
-        duration: Seconds,
-        frame_shift: Seconds,
-        sampling_rate: int,
+    duration: Seconds,
+    frame_shift: Seconds,
+    sampling_rate: int,
 ) -> int:
     """
     Compute the number of frames from duration and frame_shift in a safe way.
@@ -286,7 +330,8 @@ def compute_num_frames(
 
 def during_docs_build() -> bool:
     import os
-    return bool(os.environ.get('READTHEDOCS'))
+
+    return bool(os.environ.get("READTHEDOCS"))
 
 
 def tqdm_urlretrieve_hook(t):
@@ -330,7 +375,8 @@ def urlretrieve_progress(url, filename=None, data=None, desc=None):
     Use "desc" argument to display a user-readable string that informs what is being downloaded.
     """
     from urllib.request import urlretrieve
-    with tqdm(unit='B', unit_scale=True, unit_divisor=1024, miniters=1, desc=desc) as t:
+
+    with tqdm(unit="B", unit_scale=True, unit_divisor=1024, miniters=1, desc=desc) as t:
         reporthook = tqdm_urlretrieve_hook(t)
         return urlretrieve(url=url, filename=filename, reporthook=reporthook, data=data)
 
@@ -361,27 +407,43 @@ class nullcontext(AbstractContextManager):
 def perturb_num_samples(num_samples: int, factor: float) -> int:
     """Mimicks the behavior of the speed perturbation on the number of samples."""
     rounding = ROUND_HALF_UP if factor >= 1.0 else ROUND_HALF_DOWN
-    return int(Decimal(round(num_samples / factor, ndigits=8)).quantize(0, rounding=rounding))
+    return int(
+        Decimal(round(num_samples / factor, ndigits=8)).quantize(0, rounding=rounding)
+    )
 
 
-def compute_num_samples(duration: Seconds, sampling_rate: int, rounding=ROUND_HALF_UP) -> int:
+def compute_num_samples(
+    duration: Seconds, sampling_rate: int, rounding=ROUND_HALF_UP
+) -> int:
     """
     Convert a time quantity to the number of samples given a specific sampling rate.
     Performs consistent rounding up or down for ``duration`` that is not a multiply of
     the sampling interval (unlike Python's built-in ``round()`` that implements banker's rounding).
     """
     return int(
-        Decimal(
-            round(duration * sampling_rate, ndigits=8)
-        ).quantize(0, rounding=rounding)
+        Decimal(round(duration * sampling_rate, ndigits=8)).quantize(
+            0, rounding=rounding
+        )
     )
 
 
+def add_durations(d1: Seconds, d2: Seconds, sampling_rate: int) -> Seconds:
+    """
+    Adds two durations in a way that avoids floating point precision issues.
+    The durations in seconds are first converted to audio sample counts,
+    then added, and finally converted back to floating point seconds.
+    """
+    s1 = compute_num_samples(d1, sampling_rate=sampling_rate)
+    s2 = compute_num_samples(d2, sampling_rate=sampling_rate)
+    tot = s1 + s2
+    return tot / sampling_rate
+
+
 def compute_start_duration_for_extended_cut(
-        start: Seconds,
-        duration: Seconds,
-        new_duration: Seconds,
-        direction: Literal['center', 'left', 'right', 'random'] = 'center',
+    start: Seconds,
+    duration: Seconds,
+    new_duration: Seconds,
+    direction: Literal["center", "left", "right", "random"] = "center",
 ) -> Tuple[Seconds, Seconds]:
     """
     Compute the new value of "start" for a time interval characterized by ``start`` and ``duration``
@@ -393,13 +455,13 @@ def compute_start_duration_for_extended_cut(
         # New duration is shorter; do nothing.
         return start, duration
 
-    if direction == 'center':
+    if direction == "center":
         new_start = start - (new_duration - duration) / 2
-    elif direction == 'left':
+    elif direction == "left":
         new_start = start - (new_duration - duration)
-    elif direction == 'right':
+    elif direction == "right":
         new_start = start
-    elif direction == 'random':
+    elif direction == "random":
         new_start = random.uniform(start - (new_duration - duration), start)
     else:
         raise ValueError(f"Unexpected direction: {direction}")
@@ -427,24 +489,20 @@ def exactly_one_not_null(*args) -> bool:
 
 
 def supervision_to_frames(
-        supervision,
-        frame_shift: Seconds,
-        sampling_rate: int,
-        max_frames: Optional[int] = None
+    supervision,
+    frame_shift: Seconds,
+    sampling_rate: int,
+    max_frames: Optional[int] = None,
 ) -> Tuple[int, int]:
     """
     Utility to convert a supervision's time span into a tuple of ``(start_frame, num_frames)``.
     When ``max_frames`` is specified, it will truncate the ``num_frames`` (if necessary).
     """
     start_frame = compute_num_frames(
-        supervision.start,
-        frame_shift=frame_shift,
-        sampling_rate=sampling_rate
+        supervision.start, frame_shift=frame_shift, sampling_rate=sampling_rate
     )
     num_frames = compute_num_frames(
-        supervision.duration,
-        frame_shift=frame_shift,
-        sampling_rate=sampling_rate
+        supervision.duration, frame_shift=frame_shift, sampling_rate=sampling_rate
     )
     if max_frames:
         diff = start_frame + num_frames - max_frames
@@ -454,22 +512,14 @@ def supervision_to_frames(
 
 
 def supervision_to_samples(
-        supervision,
-        sampling_rate: int,
-        max_samples: Optional[int] = None
+    supervision, sampling_rate: int, max_samples: Optional[int] = None
 ) -> Tuple[int, int]:
     """
     Utility to convert a supervision's time span into a tuple of ``(start_sample num_samples)``.
     When ``max_samples`` is specified, it will truncate the ``num_samples`` (if necessary).
     """
-    start_sample = compute_num_samples(
-        supervision.start,
-        sampling_rate=sampling_rate
-    )
-    num_samples = compute_num_samples(
-        supervision.duration,
-        sampling_rate=sampling_rate
-    )
+    start_sample = compute_num_samples(supervision.start, sampling_rate=sampling_rate)
+    num_samples = compute_num_samples(supervision.duration, sampling_rate=sampling_rate)
     if max_samples:
         diff = start_sample + num_samples - max_samples
         if diff > 0:
@@ -492,6 +542,7 @@ def is_module_available(*modules: str) -> bool:
     https://github.com/pytorch/audio/blob/6bad3a66a7a1c7cc05755e9ee5931b7391d2b94c/torchaudio/_internal/module_utils.py#L9
     """
     import importlib
+
     return all(importlib.util.find_spec(m) is not None for m in modules)
 
 
@@ -503,7 +554,7 @@ def measure_overlap(lhs: Any, rhs: Any) -> float:
     lhs, rhs = sorted([lhs, rhs], key=lambda item: item.start)
     overlapped_area = lhs.end - rhs.start
     if overlapped_area <= 0:
-        return 0.
+        return 0.0
     dur = min(lhs.end - lhs.start, rhs.end - rhs.start)
     return overlapped_area / dur
 
